rm(list=ls())
library(AER);library(MASS);library(mvtnorm) 
library(Matrix);library(foreign)
library(xtable);library(ggplot2);
library(ggpubr);  library(dplyr)
library(readstata13);library(hdm)
library(glmnet)
set.seed(1234)

setwd("/"); if (file.exists("C:/Users/dseo5283/")){  main.dir <- "C:/Users/dseo5283/Dropbox/BinaryManyWeak/BMW_Replication/"
} else if (file.exists("/Users/dakyu/")){
  main.dir <- "/Users/dakyu/Dropbox/BinaryManyWeak/BMW_Replication/" 
} 
setwd(main.dir)
 

rbtik = dget("code/rbinary_tik_rev.R") 
 
dta =  read.dta13(file = "data/educ_sampleJOLE.dta",missing.type =TRUE)


## Exogenous variables
varsnolaginc=cbind(dta$black,dta$hispan,dta$momhsgrad,dta$momcollege,dta$dadhsgrad,dta$dadcollege,
                   dta$female,dta$agebin,dta$age2, dta$evermarrpar)
vars2at18=cbind(dta$gdp18,dta$unemp18,dta$toptax18,dta$mwage18,dta$maxben318,dta$higheredstu18,dta$rtottax18,dta$tuition18)

exo = cbind(varsnolaginc,vars2at18)
sttrend = NULL
for (jj in 1:51){
  sttrend = cbind(sttrend,dta[,which(colnames(dta) == (paste0("sttrend",jj)))])
} 

#endogenous variable
endo = cbind(dta$famincdis0to5,dta$faminc6to12, dta$faminc13to18)
#instrument
inst = cbind(dta$eitc0to5, dta$eitc6to12,dta$eitc13to18)

inst.ind = apply(inst,2,function(x){ which(is.na(x)==TRUE) })
inst.ind = unique(c(inst.ind[[1]],inst.ind[[2]],inst.ind[[3]]))

# Dependent variable: college graduation
deptmp = dta$coll
dep.ind = which(is.na(deptmp)==TRUE)
endo.ind = apply(endo,2,function(x){ which(is.na(x)==TRUE) })
endo.ind = unique(c(endo.ind[[1]],endo.ind[[2]],endo.ind[[3]]))

exo.ind = apply(exo,2,function(x){ which(is.na(x)==TRUE) })

# Remove missing obs
ind.list = unique(c(dep.ind, endo.ind, inst.ind))

statemat = model.matrix(~as.factor(dta$state[-ind.list])-1)[,-1]

dep = deptmp[-ind.list]
exo = cbind(exo[-ind.list,])

inst = inst[-ind.list,]
endo = endo[-ind.list,]
wt = dta$psidwtavg[-ind.list]/sum(dta$psidwtavg[-ind.list])

se.r1 = summary(lm(endo[,1]~exo))$residuals
se.r2 = summary(lm(endo[,2]~exo))$residuals
se.r3 = summary(lm(endo[,3]~exo))$residuals

se.r = cbind(se.r1, se.r2, se.r3)

nobs = length(dep) 
inst1 = inst
lambda.s <- lasso.s.n <- NULL
plotdat <- plotdat1 <- plotdat2 <- NULL

exind = c(1:7,10,19)
exo[,-exind] =   scale(exo[,-exind])
endo[,1:3] =   scale(endo)



#Naive Probit estimation
NaiveProbit = glm(dep~endo+exo,family = binomial(link = "probit"))


fec.hat = mean(dnorm(cbind(1,endo,exo)%*%NaiveProbit$coefficients)) 


#Outcome save
WBet <- PBet <- NULL 
test.exo = matrix(NA,nrow = 4,ncol =20 )
lam.mat = NULL

for (model in 1:2){
  if (model==1){
    inst  =     scale(inst1)
  } else if (model==2){
    inst = cbind(inst, model.matrix(~inst*statemat-1)[,-(1:(dim(inst)[2] + dim(statemat)[2]))]  )
  } 
  
  
  nexond = dim(endo)[2]
  betahat <-ape.hat<- matrix(NA,ncol = 20 , nrow = 2*(nexond))
  
  betseq = 2*(1:(nexond))-1
  veseq = betseq+1
  nexo = dim(exo)[2]
  ind = c(2:(ncol(endo)+1))
  first.vv = cbind(exo,inst)
  
  
  #Naive probit estimation results 
  
  betahat[betseq,7] = c(as.vector(summary(NaiveProbit)$coefficients[ind[1:nexond],1]) )
  betahat[veseq,7] = c(summary(NaiveProbit)$coefficients[ind[1:nexond],2] )
  betahat[betseq,8] = (abs(betahat[betseq,7]/betahat[veseq,7] ) >qnorm(0.995))+
    (abs(betahat[betseq,7]/betahat[veseq,7] ) >qnorm(0.975))+
    (abs(betahat[betseq,7]/betahat[veseq,7] ) >qnorm(0.95))
  
  ape.hat[betseq,7] =  as.vector(fec.hat*NaiveProbit$coefficients)[ind]
  
  
  
  
  #Linear probability with control
  LS_first = lm(endo~first.vv)
  fit.vv = LS_first$fitted.values
  con.vv = endo - fit.vv
  set.seed(123456)
  pcz = eigen((crossprod(inst)/nobs))     
  knz = eigen(cov(cbind(exo,inst)))  
  kappa = (sqrt(sum(knz$values^2))  )
  
  
  lamb.seq = nobs^(-.6)*(kappa)* seq(from = 1e-4, to = 1e-1, length.out = 25)
  alphatilde = cv.glmnet(x = inst, y = endo[,1], nfolds =5, intercept=T, alpha = 0,type.measure="mse",lambda = lamb.seq)$lambda.min	
  
  
  first.vv = t(t(first.vv) - colMeans(first.vv))
  fitv.tmp = first.vv%*%knz$vectors%*%diag((knz$values)/(knz$values^2 + alphatilde))%*%t(knz$vectors)%*%cov(first.vv,endo)
  conv.tmp = endo - fitv.tmp  
  
  
  sighatv = crossprod(conv.tmp)/nobs
  
  Rlinear = lm(dep~fitv.tmp+exo+conv.tmp )
  
  Rl_coef = as.vector(summary(Rlinear)$coefficients[,1])
  fec.hat = 1/mean(dnorm(Rlinear$fitted.values))
  
  startv = as.matrix(fec.hat* Rl_coef)
  
  
  ## Optimze the regularization parameter
  mse.tilde.tik <- mse.tilde.pca <-   NULL 
  for (lamtmp in lamb.seq){  
    
    
    rest = rbtik(dep=dep,endo.v=as.matrix(endo),exo.v=exo,inst.v=(inst),lambda=lamtmp,
                 start.v=startv,model="Both", interc="TRUE")
    
    psihattmp =rep(1,ncol(endo))
    
    
    
    htmp = c(dnorm(rest$ghat%*%rest$beta.nls)^2)*rest$ghat
    hfac = max(eigen(crossprod((htmp))/nobs)$values)
    hhat = as.matrix( rest$ghat[,  (ncol(cbind(endo,exo))+ind) ])%*%(psihattmp)
    sighatvv =  t(psihattmp)%*%sighatv%*%psihattmp
    pnakpha = cbind(1,exo,inst)%*%rest$kn_inv%*%t(cbind(1,exo,inst))/nobs
    htilde0 = c(crossprod(hhat)/nobs)  
    
    
    hfac2 = (1- sum(diag( pnakpha ))/nobs ) ^2
    htilde1 =  (htilde0)/hfac2 - sighatvv * sum(diag(pnakpha%*%pnakpha))/nobs
    htilde2 = sighatvv*sum(diag(pnakpha))^2/nobs
    mse.tilde.tik = append(mse.tilde.tik,  hfac*(htilde2 +htilde1)    )
    
    
    
    pcindex = (pcz$values^2 <   kappa*lamtmp)
    if (max(pcindex)==0){
      pct_z = inst
    } else{
      pck = max(1,(which.max(pcindex)-1))
      pct_z = inst%*%pcz$vectors[,1:pck]
    }
    
    
    rest = rbtik(dep=dep,endo.v=as.matrix(endo),exo.v=exo,inst.v=as.matrix(pct_z),lambda=0,
                 start.v=startv,model="Both", interc="F")
    
    
    psihattmp = rep(1, ncol(endo))
    htmp = c(dnorm(rest$ghat%*%rest$beta.nls)^2)*rest$ghat
    hfac = max(eigen(crossprod((htmp))/nobs)$values)
    hhat = (rest$ghat[, (ncol(cbind(endo,exo))+ind) ])%*%(psihattmp)
    sighatvv =  t(psihattmp)%*%sighatv%*%psihattmp
    pnakpha = cbind(exo, pct_z)%*%rest$kn_inv%*%t(cbind(exo, pct_z))/nobs
    htilde0 = c(crossprod(hhat)/nobs)  
    
    
    hfac2 = (1- sum(diag( pnakpha ))/nobs ) ^2
    htilde1 =  (htilde0)/hfac2 - sighatvv * sum(diag(pnakpha%*%pnakpha))/nobs
    htilde2 = sighatvv*sum(diag(pnakpha))^2/nobs
    mse.tilde.pca = append(mse.tilde.pca,  hfac*(htilde2 +htilde1)    )
  }
  
  
  
  lambda.tik = lamb.seq[which.min(mse.tilde.tik)]
  lambda.pca = lamb.seq[which.min(mse.tilde.pca)]
  lam.mat.tmp = c(lambda.tik, lambda.pca)
  lam.mat = rbind(lam.mat, lam.mat.tmp)
  print(lambda.tik);print(lambda.pca)
  
  startmat = (dim(exo)[2]+2*dim(endo)[2])
  startmat = rbind(1,matrix(runif(3*startmat),ncol=3))
  
  ### Tikhonov regularized linear probability model; not reported in the paper but used as the initialization parameter
  first.vv = cbind(1,exo,inst)
  eig.first = eigen(crossprod(first.vv)/nobs)
  egval = sqrt(eig.first$values^2)
  kn_inv =  (eig.first$vectors)%*%diag(egval/(egval^2 + lambda.tik))%*%t(eig.first$vectors)
  
  firstbet = kn_inv%*%(t(first.vv)%*%endo/nobs)
  
  Fit.v = first.vv%*%firstbet
  Con.v = endo-Fit.v
  
  
  ncc = nexo + ncol(inst)+1 
  Rlinear = lm(dep~Fit.v+exo+Con.v )
  
  Rl_coef = as.vector(summary(Rlinear)$coefficients[,1])
  v_tsls = vcovHC(Rlinear,"HC1")
  
  
  ve_coeff = sqrt(diag( v_tsls))
  coeff =  Rl_coef
  betahat[betseq,11] = (coeff)[ind] 
  ape.hat[betseq,11] =  (coeff)[ind] 
  betahat[veseq,11] = ve_coeff[ind]
  betahat[betseq,12] = (abs(betahat[betseq,11]/betahat[veseq,11] ) >qnorm(0.995))+
    (abs(betahat[betseq,11]/betahat[veseq,11] ) >qnorm(0.975))+
    (abs(betahat[betseq,11]/betahat[veseq,11] ) >qnorm(0.95))
  
  
  rm(Rl_coef)
  rm(Fit.v)
  rm(Con.v)
  rm(firstbet)
  rm(v_tsls)
  
  startv = as.matrix(coeff)
  
  
  ### PCA regularized estimator 
  ei_inst = eigen(cov((inst)))
  pct_z = (inst)%*%ei_inst$vectors[,1:max(max(1,(which.max(ei_inst$values^2 <= lambda.pca))), dim(endo)[2])]
  
  rest = rbtik(dep = dep, endo.v = endo, exo.v = exo, inst.v = pct_z, 0, 
               start.v = as.matrix(coeff), model="Both", interc = "TRUE")
  
  fec.hat = mean(dnorm(rest$ghat%*%rest$beta.mle))
  
  tmpind = ncol(cbind(endo,exo))+ind
  
  iotavec = matrix(0,nrow = length(rest$beta.mle), ncol = ncol(endo))
  for (jjj in 1:ncol(endo)){
    iotavec[ind[jjj],jjj] = -1
    iotavec[tmpind[jjj],jjj]=1
  }
  
  psihat = t(iotavec)%*%as.vector(rest$beta.mle) 
  test.exo[(2*model-1),17] =    t(psihat)%*%solve(t(iotavec)%*%rest$var.mle%*%iotavec) %*%psihat
  test.exo[(2*model),17] =  1-  pchisq( test.exo[(2*model-1),17],ncol(endo))
  
  
  #SCRCMLE
  ve_coeff = sqrt(diag(rest$var.mle)[ind])
  betahat[betseq,17] =  as.vector(rest$beta.mle)[ind]
  ape.hat[betseq,17] =  as.vector(fec.hat*rest$beta.mle)[ind]
  betahat[veseq,17] = ve_coeff 
  betahat[betseq,18] = (abs(betahat[betseq,17]/betahat[veseq,17] ) >qnorm(0.995))+
    (abs(betahat[betseq,17]/betahat[veseq,17] ) >qnorm(0.975))+
    (abs(betahat[betseq,17]/betahat[veseq,17] ) >qnorm(0.95))
  
  
  
   
  fec.hat = mean(dnorm(rest$ghat%*%rest$beta.nls))
  
  #SCRCNLSE
  psihat = t(iotavec)%*%as.vector(rest$beta.nls) 
  test.exo[(2*model-1),19] =  t(psihat)%*%solve(t(iotavec)%*%rest$var.nls%*%iotavec) %*%psihat
  test.exo[(2*model),19] =   1-  pchisq( test.exo[(2*model-1),19],ncol(endo))
  
  ve_coeff =  sqrt(diag(rest$var.nls)[ind])
  betahat[betseq,19] =  as.vector(rest$beta.nls)[ind]
  ape.hat[betseq,19] =  as.vector(fec.hat*rest$beta.nls)[ind]
  betahat[veseq,19] = ve_coeff
  betahat[betseq,20] = (abs(betahat[betseq,19]/betahat[veseq,19]) >qnorm(0.975))+
    (abs(betahat[betseq,19]/betahat[veseq,19]) >qnorm(0.995))+
    (abs(betahat[betseq,19]/betahat[veseq,19]) >qnorm(0.95))
  
  
  
  #Tikhonov regularized estimation
  
  rest = rbtik(dep = dep, endo.v = endo, exo.v = exo, inst.v = inst, lambda.tik, start.v = as.matrix(coeff), model="Both", interc = "TRUE")
  fec.hat = mean(dnorm(rest$ghat%*%rest$beta.mle))
  
  psihat = t(iotavec)%*%as.vector(rest$beta.mle) 
  test.exo[(2*model-1),13] = t(psihat)%*%solve(t(iotavec)%*%rest$var.mle%*%iotavec) %*%psihat
  test.exo[(2*model),13] = 1-  pchisq( test.exo[(2*model-1),13],ncol(endo))
  
  ve_coeff = sqrt(diag(rest$var.mle)[ind])
  betahat[betseq,13] = as.vector(rest$beta.mle)[ind]
  ape.hat[betseq,13] =  as.vector(fec.hat*rest$beta.mle)[ind]
  betahat[veseq,13] = ve_coeff 
  betahat[betseq,14] = (abs(betahat[betseq,13]/betahat[veseq,13] ) >qnorm(0.995))+
    (abs(betahat[betseq,13]/betahat[veseq,13] ) >qnorm(0.975))+
    (abs(betahat[betseq,13]/betahat[veseq,13] ) >qnorm(0.95))
  
  
  fec.hat = mean(dnorm(rest$ghat%*%rest$beta.nls))
  
  psihat = t(iotavec)%*%as.vector(rest$beta.nls) 
  
  test.exo[(2*model-1),15] = t(psihat)%*%solve(t(iotavec)%*%rest$var.nls%*%iotavec) %*%psihat
  test.exo[(2*model),15] =  1-  pchisq( test.exo[(2*model-1),15],ncol(endo))
  
  ve_coeff = sqrt(diag(rest$var.nls )[ind])
  betahat[betseq,15] = as.vector(rest$beta.nls)[ind]
  ape.hat[betseq,15] =  as.vector(fec.hat*rest$beta.nls)[ind]
  betahat[veseq,15] = ve_coeff
  betahat[betseq,16] = (abs(betahat[betseq,15]/betahat[veseq,15]) >qnorm(0.975))+
    (abs(betahat[betseq,15]/betahat[veseq,15]) >qnorm(0.995))+
    (abs(betahat[betseq,15]/betahat[veseq,15]) >qnorm(0.95))
  
  
  
  ###Rivers&Vuong Estimator
  RiversVuong = rbtik(dep = dep, endo.v = endo, exo.v = exo, inst.v = inst, 0, start.v = as.matrix(coeff), model="Both", interc = "TRUE")
  fec.hat = mean(dnorm(RiversVuong$ghat%*%RiversVuong$beta.mle))
  
  psihat = t(iotavec)%*%as.vector(RiversVuong$beta.mle) 
  test.exo[(2*model-1),5] = t(psihat)%*%solve(t(iotavec)%*%RiversVuong$var.mle%*%iotavec) %*%psihat
  test.exo[(2*model),5] = 1-  pchisq( test.exo[(2*model-1),5],ncol(endo))
  
  ve_coeff = sqrt(diag(RiversVuong$var.mle)[ind])
  betahat[betseq,5] = as.vector(RiversVuong$beta.mle)[ind]
  ape.hat[betseq,5] =  as.vector(fec.hat*RiversVuong$beta.mle)[ind]
  betahat[veseq,5] = ve_coeff 
  betahat[betseq,6] = (abs(betahat[betseq,5]/betahat[veseq,5] ) >qnorm(0.995))+
    (abs(betahat[betseq,5]/betahat[veseq,5] ) >qnorm(0.975))+
    (abs(betahat[betseq,5]/betahat[veseq,5] ) >qnorm(0.95))
  
  
  
  ##Lasso estimation
  library(hdm)
  rlas = rlassoIV(y = dep, x = exo, d=endo,z=inst, penalty = list(homoscedastic = F),select.X=FALSE, select.Z=TRUE)
  endo.fit <- con.fit<- NULL
  for (jj in 1:ncol(endo)){
    s.first = cbind(inst,exo)[,rlas$selected[,jj]]
    endo.fit = cbind(endo.fit, predict(lm(endo[,jj]~s.first)))
    con.fit = cbind(con.fit,endo[,jj]-endo.fit[,jj] )
  }
  
  
  
  rlass = lm(dep~endo.fit+exo+con.fit)
  vrlass = vcovHC(rlass, "HC0")
  
  
  psihat = t(iotavec)%*%as.vector(rlass$coefficients)
  test.exo[(2*model-1),1] = t(psihat)%*%solve(t(iotavec)%*%vrlass%*%iotavec)%*%psihat 
  test.exo[(2*model),1] =1-pchisq( test.exo[(2*model-1),1] , ncol(endo))
  
  ve_coeff = sqrt(diag( vrlass))
  coeff =  rlass$coefficients
  betahat[betseq,1] = (coeff)[ind]
  ape.hat[betseq,1] =  (coeff)[ind]
  betahat[veseq,1] = ve_coeff[ind]
  betahat[betseq,2] = (abs(betahat[betseq,1]/betahat[veseq,1] ) >qnorm(0.995))+
    (abs(betahat[betseq,1]/betahat[veseq,1] ) >qnorm(0.975))+
    (abs(betahat[betseq,1]/betahat[veseq,1] ) >qnorm(0.95))
  
  lasso.s.n = rbind(lasso.s.n,(colSums(rlas$selected)))
  
  
  ### Computation of APE 
  exomat = t(matrix(rep(colMeans(exo),nobs), ncol = nobs))
  length.tmpseq = 50
  seqendo = 1:ncol(endo)
  for (jtmp in 1:ncol(endo)){
    min.jtmp = min(seqendo[-jtmp])
    max.jtmp = max(seqendo[-jtmp])
    tmpend1 = median(endo[,min.jtmp])
    tmpend2 = median(endo[,max.jtmp])
    tmpseq = seq(-1,1,length.out = length.tmpseq)
    
    tmpmat = matrix(NA, nrow = length.tmpseq, ncol = ncol(endo))
    tmpmat[,jtmp] = tmpseq
    tmpmat[,min.jtmp] = tmpend1
    tmpmat[,max.jtmp] = tmpend2
    
    data <- data.frame(cbind(tmpmat,exomat[1:length.tmpseq,]))
    
    names(data) <- c("EITC1","EITC2","EITC3",colnames(exo))
    asf_mle <- apf_mle <- cbind(tmpseq,NA) 
    asf_rv <- apf_rv <-  cbind(tmpseq,NA) 
    
    for(i in 1:dim(asf_mle)[1]){
      tmpmat = matrix(NA, nrow = nobs, ncol = ncol(endo))
      tmpmat[,jtmp] = c(asf_mle[i,1])
      tmpmat[,min.jtmp] = tmpend1
      tmpmat[,max.jtmp] = tmpend2
      
      
      dat1 <- data.frame(cbind(1, tmpmat,exomat,rest$ghat[,tmpind] ),row.names = NULL)
      dat2 <- data.frame(cbind(1, tmpmat,exomat,rest$ghat[,tmpind]),row.names = NULL)
      names(dat1) <-names(dat2) <- c("int","EITC1","EITC2","EITC3",colnames(exo))
      asf_mle[i,2] <- mean(pnorm(  as.matrix(dat1)%*%(as.vector(rest$beta.mle)) )) 
      asf_rv[i,2] <- mean(pnorm(  as.matrix(dat2)%*%(as.vector(RiversVuong$beta.mle)) )) 
      apf_mle[i,2] <-  mean(dnorm(  as.matrix(dat1)%*%(as.vector(rest$beta.mle)) ))*c(rest$beta.mle[ind[3]])
      apf_rv[i,2] <-  mean(dnorm(  as.matrix(dat2)%*%(as.vector(RiversVuong$beta.mle)) ))*c(RiversVuong$beta.mle[ind[3]])
    } 
    
    asf_pro <- apf_pro <- cbind(tmpseq,NA)
    for(i in 1:dim(asf_mle)[1]){
      tmpmat = matrix(NA, nrow = nobs, ncol = ncol(endo))
      tmpmat[,jtmp] = c(asf_mle[i,1])
      tmpmat[,min.jtmp] = tmpend1
      tmpmat[,max.jtmp] = tmpend2
      dat <- data.frame(cbind(1, tmpmat,exomat), row.names = NULL)
      names(dat) <- c("int","EITC1","EITC2","EITC3",colnames(exo))
      asf_pro[i,2] <- mean(pnorm(  as.matrix(dat)%*%NaiveProbit$coefficients) ) 
      apf_pro[i,2] <- mean(dnorm(  as.matrix(dat)%*%NaiveProbit$coefficients) )*c(NaiveProbit$coefficients[ind[3]])
    }
    
    
    pltmp <- (rbind(cbind(model,jtmp,data$EITC1,data$EITC2,data$EITC3,asf_mle[,2],"TRCMLE"), 
                    cbind(model,jtmp,data$EITC1,data$EITC2,data$EITC3,asf_rv[,2],"2SCMLE"),
                    cbind(model,jtmp,data$EITC1,data$EITC2,data$EITC3,asf_pro[,2],"Probit")))
    ape.tmp <- (rbind(cbind(model,jtmp,data$EITC1,data$EITC2,data$EITC3,apf_mle[,2],"TRCMLE"), 
                      cbind(model,jtmp,data$EITC1,data$EITC2,data$EITC3,apf_rv[,2],"2SCMLE"),
                      cbind(model,jtmp,data$EITC1,data$EITC2,data$EITC3,apf_pro[,2],"Probit")))
    plotdat = rbind(plotdat,pltmp)
    plotdat1 = rbind(plotdat1,ape.tmp)    
  }
  
  
  
  lambda.s = rbind(lambda.s,c(lambda.tik,lambda.pca))
  
  WBet = rbind(WBet, betahat)
  PBet = rbind(PBet, ape.hat)
  
}

print("lambda")
print(lam.mat)
 

pl.list = list()  

for (md in 1:3){ 
  PDat = data.frame(plotdat[which(plotdat[,2]==md),])
  names(PDat) <- c("Model","md","I1","I2","I3","E","Estimator") 
  PDat$endo.v <- as.numeric(as.character(PDat[,2+md]))
  PDat$y <- as.numeric(as.character(PDat$E))
  
  
  for (model in 1:2){
    
    pdat = PDat[as.numeric(PDat$Model)==model,]
    pl =  ggplot(pdat, aes(x=endo.v, y=y, colour = Estimator, group=Estimator)) + 
     geom_line(data = pdat ,aes(linetype=Estimator ) ,size = 1.2) + 
      scale_x_continuous(name=expression('') , n.breaks =3  )+
      scale_y_continuous(name = expression(' '), n.breaks =3 ) + 
      coord_cartesian(  ylim = c(0,.5) ) +
      theme_bw() + 
      scale_linetype_manual(values=c("dotted","solid","dashed" ) )+
      scale_color_manual( values=c("grey70","grey50", "grey30"))+
      theme(axis.title.x = element_text(size = rel(1.3), angle = 0,  face = "italic" ), 
            axis.title.y = element_text(size = rel(1.3), angle = 90,   face = "italic" ),
            plot.title = element_text(size = rel(1.3),   face = "bold",hjust = -0.1),
            plot.subtitle = element_text(size = rel(1.2), face = "italic",hjust = -0.1),
            axis.text = element_text(size = rel(1)), 
            axis.ticks.length.y = unit(.25, "cm"),
            axis.ticks.length.x = unit(-.25, "cm"),
            axis.text.x = element_text(margin = margin(t = .3, unit = "cm")),
            legend.text = element_text(size=rel(1.3),  face = "bold"),
            legend.title = element_text(size = rel(1.3),  face = 
                                          "bold"), legend.key.width = unit(2,"cm"),  )
    
    pl.list[[(2*(md-1)+model)]] = pl
  }
}



gg.asf = ggarrange(pl.list[[1]],pl.list[[3]], pl.list[[5]],pl.list[[2]], pl.list[[4]], pl.list[[6]] , ncol=3,nrow=2, common.legend = TRUE ,legend = "bottom") 
ggsave("result/eitc_21dec17_jae.jpeg",plot=gg.asf,width=10, height=4,dpi=1200 )
while (!is.null(dev.list()))  dev.off()



apl.list = list()  
for (md in 1:3){ 
  PDat = data.frame(plotdat1[which(plotdat1[,2]==md),])
  names(PDat) <- c("Model","md","I1","I2","I3","E","Estimator") 
  PDat$endo.v <- as.numeric(as.character(PDat[,2+md]))
  PDat$y <- as.numeric(as.character(PDat$E))
  
  
  for (model in 1:2){
    
    pdat = PDat[as.numeric(PDat$Model)==model,]
    pl =  ggplot(pdat, aes(x=endo.v, y=y, colour = Estimator, group=Estimator)) + 
        geom_line(data = pdat ,aes(linetype=Estimator ) ,size = 1.2) + 
      scale_x_continuous(name=expression('') , n.breaks =3  )+
      scale_y_continuous(name = expression(' '), n.breaks =3 ) + 
      coord_cartesian(  ylim = c(0,1) ) +
       theme_bw() + 
      scale_linetype_manual(values=c("dotted","solid","dashed" ) )+
      scale_color_manual( values=c("grey70","grey50", "grey30"))+
      theme(axis.title.x = element_text(size = rel(1.3), angle = 0, face = "italic" ), 
            axis.title.y = element_text(size = rel(1.3), angle = 90, face = "italic" ),
            plot.title = element_text(size = rel(1.3),  face = "bold",hjust = -0.1),
            plot.subtitle = element_text(size = rel(1.2), face = "italic",hjust = -0.1),
            axis.text = element_text(size = rel(1)), 
            axis.ticks.length.y = unit(.25, "cm"),
            axis.ticks.length.x = unit(-.25, "cm"),
            axis.text.x = element_text(margin = margin(t = .3, unit = "cm")),
            legend.text = element_text(size=rel(1.3),  face = "bold"),
            legend.title = element_text(size = rel(1.3),  face = 
                                          "bold"), legend.key.width = unit(2,"cm"),  )
    
   apl.list[[(2*(md-1)+model)]] = pl
  }
}



gg.asf = ggarrange(apl.list[[1]],apl.list[[3]], apl.list[[5]],apl.list[[2]], apl.list[[4]], apl.list[[6]] , ncol=3,nrow=2, common.legend = TRUE ,legend = "bottom") 
ggsave("result/eitc_apf_21dec17_jae.jpeg",plot=gg.asf,width=10, height=4,dpi=1200 )
while (!is.null(dev.list()))  dev.off()



WBett = WBet[(1:6), c(13,17,1,5)]
WBettt = xtable(WBett, digits = 4)

sigalpha=.05

report.func = function(x, digits=4){ 
  x = as.matrix(x)
  z.sig.num = rep("{}", ceiling(nrow(x)/2) )
  odd.seq = seq(1,by=2,length.out = ceiling(nrow(x)/2))
  even.seq = odd.seq+1
  xx = abs(c(x[odd.seq,])/c(x[even.seq,]))
  z.sig.num[which(abs(xx)>=qnorm(0.995))] = "{***}"
  z.sig.num[which(abs(xx)>=qnorm(0.975)&abs(xx)<qnorm(0.995))] = "{**}"
  z.sig.num[which(abs(xx)>=qnorm(0.95)&abs(xx)<qnorm(0.975))] = "{*}"
  rtn <- matrix(NA, nrow = nrow(x), ncol =1)
  rtn[odd.seq,] <-  paste0(formatC(x[odd.seq,], digits,format="f"),"$^",z.sig.num,"$") 
  rtn[even.seq,] <- paste0("(",formatC(x[even.seq,], digits,format="f"),")")
  return(rtn)
}

df = data.frame(apply(WBet[1:6, c(13,17,5,1,7)],2,report.func)   )
colnames(df) <- c("TRCMLE", "SCRCMLE", "2SCMLE", "Lasso", "Probit")
df%>%xtable()%>%print.xtable(type="latex",file="result/table3_betaestimate_model1.txt",sanitize.text.function = function(x){x},include.rownames=FALSE)

df = data.frame(apply(WBet[7:12, c(13,17,5,1,7)],2,report.func)   )
colnames(df) <- c("TRCMLE", "SCRCMLE", "2SCMLE", "Lasso", "Probit")
df%>%xtable()%>%print.xtable(type="latex",file="result/table3_betaestimate_model2.txt",sanitize.text.function = function(x){x},include.rownames=FALSE)


df = data.frame(apply(test.exo[, c(13,17,5,1,7)],2,report.func)   )
colnames(df) <- c("TRCMLE", "SCRCMLE", "2SCMLE", "Lasso", "Probit")
df = cbind(c("Model 1", "Model 2"), df[c(1,3),])
df%>%xtable()%>%print.xtable(type="latex",file="result/table3_exotest.txt", sanitize.text.function = function(x){x},include.rownames=FALSE)

report.func1 = function(x, digits=4){  
  rtn  <-  paste0(formatC(x, digits,format="f")) 
  return(rtn)
}
df = data.frame(apply(cbind(PBet[,c(13,17,5,1)], ape.hat[,7]),2,report.func1))


df%>%xtable()%>%print.xtable(type="latex",file="result/ape_table.txt", sanitize.text.function = function(x){x},include.rownames=FALSE)

