##################################
# R Code to compute the RCMLE and RNLSE based on Tikhonov regularization #
# dep : dependent variable  
# endo.v: endogenous variable (vector) 
# exo.v: exogenous variables (matrix)
# inst.v: instrumental variables (matrix)
# lambda: regularization parameter
# start.v: starting values
# model: report the RCMLE if model=="MLE" ; the RCMLE and RNLSE if model=="Both" 
# interc==TRUE if intercept term is needed  
##################################


rbinary = function(dep,endo.v,exo.v,inst.v,lambda,start.v,model,interc){
  
  first_v = cbind(exo.v , inst.v)
  if (as.factor(interc)==TRUE){
    first_v = cbind(1,first_v)
  }
     nl_function = function(input_vec, coef_vec){
      return(pnorm(input_vec%*%coef_vec))
    }
 
     
      varestfun = function(dep,ghat,thetahat,thetahat.nls,inst,alpha,psi_hat,psi_hat.nls,kn_inv){
       nn = nrow(ghat)
       fit.v = ghat%*%(thetahat)
       fit.p = as.vector(pnorm(fit.v)*(1-pnorm(fit.v)))
       fit.d = as.vector(dnorm(fit.v))
       yphisq = (as.vector((dep-pnorm(fit.v))^2))
       
       fit.v.nls = ghat%*%(thetahat.nls)
       fit.p.nls = as.vector(pnorm(fit.v.nls)*(1-pnorm(fit.v.nls)))
       fit.d.nls = as.vector(dnorm(fit.v.nls))
       yphisq.nls = (as.vector((dep-pnorm(fit.v.nls))^2))
       
       
       phisq.nls = (fit.d.nls^2)
       phisq2.nls = phisq.nls*yphisq.nls 
       gam1hat.nls = t(ghat)%*%(phisq.nls*ghat)
       J1hat.nls = t(ghat)%*%(phisq2.nls*ghat)
       nu3hat.nls =  t(ghat )%*%(phisq.nls*inst) 
       
       
       phisq = (fit.d^2/fit.p^2)*yphisq
       gam1hat = t(ghat)%*%(phisq*ghat)
       nu3hat = t(ghat )%*%(phisq*inst) 
 
       
        ntmpind = (dim(ghat)[2] - length(psi_hat.nls)+1):(dim(ghat)[2])
       nu0hat.nls = as.matrix(ghat[,ntmpind])%*%as.vector(psi_hat.nls)
       nu0hat  = as.matrix(ghat[,ntmpind])%*%as.vector(psi_hat)
       
       
       
       nu1hat = c(crossprod(nu0hat)/nn) * kn_inv 
       nu1hat.nls = c(crossprod(nu0hat.nls)/nn) * kn_inv
       
       J2hat1 =  (nu3hat/nn)%*%nu1hat%*%t(nu3hat/nn)
       J2hat1.nls =  (nu3hat.nls/nn)%*%nu1hat.nls%*%t(nu3hat.nls/nn)
       
       gam2hat = gam1hat/nn + J2hat1
       gam2hat.nls = J1hat.nls/nn + J2hat1.nls
       
       var.nls = solve(gam1hat.nls/nn)%*%gam2hat.nls%*%solve(gam1hat.nls/nn)
       var.mle = solve(gam1hat/nn)%*%gam2hat%*%solve(gam1hat/nn)
       
       var.mle = var.mle/nn
       var.nls = var.nls/nn
       var_list = list("var.nls" = var.nls, "var.mle" = var.mle)
       return(var_list)
     }
     
      nn = nrow(first_v)
      eigen_z = eigen(crossprod(first_v)/nn)
      egval = sqrt(eigen_z$values^2)
      kn_inv =  (eigen_z$vectors)%*%diag(egval/(egval^2 + lambda))%*%t(eigen_z$vectors)
 firstbet = kn_inv%*%(t(first_v)%*%endo.v/nn)
  
  fit.v = first_v%*%firstbet
  con.v = endo.v-fit.v
   
  if (is.null(exo.v) == TRUE){
    g_hat = cbind(1, fit.v, con.v) 
    pro_model = glm(dep~fit.v+con.v,family=binomial(link="probit"), glm.control(maxit = 50))
  }else{
    g_hat = cbind(1, fit.v, exo.v, con.v) 
    pro_model = glm(dep~fit.v+exo.v+con.v,family=binomial(link="probit"), glm.control(maxit = 50))
  }
  
  
  coefs.probit = pro_model$coefficients
  ntmpind = (length(coefs.probit) - ncol(as.matrix(endo.v))+1):(length(coefs.probit))
  psihat = coefs.probit[ntmpind]-coefs.probit[2:(ncol(as.matrix(endo.v))+1  ) ]
  
   
    gtmp = NULL
    coefs.nls = NULL
    contemp = NULL
    options(warn = -1)
    for (jjj in 1:ncol(start.v)){    
     
    nls_model = nls(dep~nl_function(g_hat,coef),
                    start = list(coef=start.v[,jjj]), 
                    control = nls.control(maxiter = 50, tol = 1e-4, warnOnly = TRUE))
    contemp = append(contemp,nls_model$convInfo$stopMessage)
    if (contemp[jjj]=="converged"){
    gtmp = append(gtmp,summary(nls_model)$sigma)
    coefs.nls = cbind(coefs.nls,summary(nls_model)$coefficients[,1] )
    } else{
      gtmp = append(gtmp,NA)
      coefs.nls = cbind(coefs.nls,rep(NA,length(coefs.probit)))
    }
    } 
 
    
    options(warn = 0)
    
    if ( length(which.min(gtmp))==0  ){
      stop("Convergence Failed")
    } else if (contemp[which.min(gtmp)]!= "converged"){
      stop("Convergence Failed")
    } 
    
    
    coefs.nls = coefs.nls[,which.min(gtmp)]
    psihat.nls = coefs.nls[ntmpind]-coefs.nls[2:(ncol(as.matrix(endo.v))+1  ) ]
     
    ve = varestfun(dep=dep,ghat=g_hat,thetahat=coefs.probit,thetahat.nls = coefs.nls,inst = first_v,alpha = lambda, 
                psi_hat = psihat, psi_hat.nls = psihat.nls,kn_inv)
  
  if (model=="MLE"){
  res_list = list("beta.mle"=coefs.probit, "var.mle"=ve$var.mle )
  } else {
  res_list = list("beta.mle"=coefs.probit,"beta.nls" = coefs.nls, "var.mle"=ve$var.mle, "var.nls" = ve$var.nls, "ghat" = g_hat, "kn_inv" = kn_inv, "pihat" = firstbet )
  }
return(res_list)
  
}